"""pytest fixtures x-files
"""
from pathlib import Path
import pytest


@pytest.fixture(scope="function")
def function_xfile(request):
    func_name = request.function.__name__
    test_name = func_name.split("_", 1)[-1]
    module_path = request.module.__file__
    return Path(module_path).with_suffix(
        ".{test_name}._x_".format(test_name=test_name))


@pytest.fixture(scope="module")
def module_xfile(request):
    module_path = Path(request.module.__file__)
    return module_path.with_suffix("._x_")


@pytest.fixture(scope="module")
def package_xfile(request):
    module_path = Path(request.module.__file__)
    return module_path.with_name("__init__._x_")


@pytest.fixture(scope="function")
def function_yaml(function_xfile):
    path = function_xfile.with_suffix(".yaml")
    with path.open(encoding="utf-8") as f:
        import yaml
        return yaml.load(f)


@pytest.fixture(scope="module")
def module_yaml(module_xfile):
    path = module_xfile.with_suffix(".yaml")
    with path.open(encoding="utf-8") as f:
        import yaml
        return yaml.load(f)


@pytest.fixture(scope="module")
def package_yaml(package_xfile):
    path = package_xfile.with_suffix(".yaml")
    with path.open(encoding="utf-8") as f:
        import yaml
        return yaml.load(f)


@pytest.fixture(scope="function")
def function_json(function_xfile):
    path = function_xfile.with_suffix(".json")
    with path.open(encoding="utf-8") as f:
        import json
        return json.load(f)


@pytest.fixture(scope="module")
def module_json(module_xfile):
    path = module_xfile.with_suffix(".json")
    with path.open(encoding="utf-8") as f:
        import json
        return json.load(f)


@pytest.fixture(scope="module")
def package_json(package_xfile):
    path = package_xfile.with_suffix(".json")
    with path.open(encoding="utf-8") as f:
        import json
        return json.load(f)



import pytest


def test_happy(function_yaml):
    print("function_yaml in test #1", function_yaml)


def test_list(function_yaml):
    print("function_yaml in test #2", function_yaml)


def test_indir(blob):
    print("blob-indir", blob)


def test_module_yaml(module_yaml):
    print("module:", module_yaml)


def test_package_yaml(package_yaml, function_yaml):
    print("module:", package_yaml)
    print("function:", function_yaml)



"""Doc test is good.
"""


def test_happy(function_yaml):
    print(" in test #1", function_yaml)


def test_list(function_yaml, module_yaml):
    print("scenario in test #2:fun:", function_yaml)
    print("scenario in test #2:module:", module_yaml)



import requests


def test_get(function_yaml):
    url = function_yaml["url"]
    resp = requests.get(url)
    assert resp.ok
    data = resp.json()
    expected = function_yaml["expected"]
    assert data["args"] == expected["args"]
    exp_headers = expected["headers"]
    data_headers = data["headers"]
    # check presence of headers
    for header_name in exp_headers["exact_value"]:
        assert header_name in data_headers
    for header_name in exp_headers["prefix_match"]:
        assert header_name in data_headers
    # check values of headers
    for header_name, exp_value in exp_headers["exact_value"].items():
        assert data_headers[header_name] == exp_value
    # check prefix
    for header_name, exp_value in exp_headers["prefix_match"].items():
        assert data_headers[header_name].startswith(exp_value)


def test_status(function_yaml):
    url_pattern = function_yaml["url"]
    for code, exp_reason in function_yaml["codes"].items():
        url = url_pattern.format(code=code)
        resp = requests.get(url)
        assert resp.status_code == code
        assert resp.reason == exp_reason



"""Docstring is good.
"""


def test_happy(function_xfile, module_xfile, package_xfile):
    print("function_xfile", function_xfile)
    print("module_xfile", module_xfile)
    print("package_xfile", package_xfile)


def test_jsonfile(function_json, module_json, package_json):
    print("function_json", function_json)
    print("module_json", module_json)
    print("package_json", package_json)
